from typing import Tuple
from lib import reg
from lib.parsing.parser import Parser, ParsingError


def get_task(string: str) -> Tuple[str, str]:
    assert '-' in string, "Teacher solution could not be parsed correctly."
    teacher_type, task = string.split("-", 1)
    if '-' in task:
        task, _ = task.split("-", 1)
    assert len(teacher_type) != 0, "Teacher solution code must not ne empty"
    assert len(task) != 0, "Task code must not be empty"

    return (teacher_type, task)


def dfa_transform(string: str, automaton_type: str) -> reg.DFA:
    try:
        parser = Parser()
        if automaton_type in {"DFA", "TOT", "MIN", "TOC", "MIC"}:
            automaton = parser.str_to_dfa(string)
        elif automaton_type == "NFA":
            automaton = parser.str_to_nfa(string).determinize()
        elif automaton_type == "EFA":
            automaton = parser.str_to_nfa(string).eliminate_epsilon().determinize()
        elif automaton_type == "GRA":
            automaton = parser.str_to_reggrammar(string).reggrammar_to_nfa().determinize()
        elif automaton_type == "REG":
            automaton = parser.str_to_regex(string).regex_to_efa().eliminate_epsilon().determinize()

        return automaton

    except ParsingError as ex:
        raise ParsingError(ex.args)


def nfa_transform(string: str, automaton_type: str) -> reg.NFA:
    try:
        parser = Parser()
        return parser.str_to_nfa(string)

    except ParsingError as ex:
        raise ParsingError(ex.args)


def check_task(dfa: reg.DFA, task: str) -> str:
    output = ""
    if task not in {"TOT", "MIN", "TOC", "MIC"}:
        return output

    if task == "TOT" and not dfa.is_total():
        output = "DFA není totální."

    if task == "MIN" and not dfa.is_minimal():
        output = "DFA není minimální."

    canonical = dfa.is_canonical()
    if task == "CAN" and not canonical:
        output = "DFA není kanonický."

    if task == "TOC" and (not dfa.is_total() or not canonical):
        if not dfa.is_total():
            output = "DFA není totální. "
        if not canonical:
            output += "DFA není kanonický."

    if task == "MIC" and (not dfa.is_minimal() or not canonical):
        if not dfa.is_minimal():
            output = "DFA není minimální. "
        if not canonical:
            output += "DFA není kanonický."

    return output

def check_empty(student_solution, teacher_solution):
    if teacher_solution.is_empty() and student_solution.is_empty():
        exit_correct()


def check_alphabets(student_alpha, teacher_alpha, task: str) -> str:
    output = ""
    if student_alpha != teacher_alpha:
        output = "Abecedy zadaného jazyka a studentova řešení se liší"

        if task == "REG":
            output += " (abeceda je odvozena ze všech písmen objevujících se v regulárním výrazu)."
        elif task == "GRA":
            output += " (abeceda je odvozena ze všech terminálů objevujících se v pravidlech gramatiky)."
        else:
            output += " (abeceda je odvozena ze všech písmen objevujících se v přechodové funkci)."

        # TODO ask also print both alphabets? Or difference?
        #print("Studentova abeceda:" + reg.Parser.names_to_str(student_alpha))
        #print("Abeceda zadaného jazyka:" + reg.Parser.names_to_str(teacher_alpha))

    return output

def exit_correct():
    print("Správné řešení.")
    exit(0)


def exit_incorrect():
    print("Nesprávné řešení.")
    exit(1)